﻿using System;
using System.Windows.Media;
using Styx.Common;
using Styx.CommonBot;
using Styx.Plugins;
using Styx.WoWInternals;
//using Styx.Common.BehaviorTree;

namespace Styx
{
    public class LogMark : HBPlugin
    {
        public override string Name { get { return "Log Marking Utility"; } }
        public override string Author { get { return "Bobby53"; } }
        public override Version Version { get { return _version; } }
        private readonly Version _version = new Version(1, 0, 0, 2);

        private bool haveInitialized = false;
        private BotEvents.OnBotStartDelegate botStart;
        private BotEvents.OnBotStopDelegate botStop;

        
        public override  void Initialize()
        {
            if ( haveInitialized )
                return;

            Logging.WriteDiagnostic("[LOGMARK] Initialize");
            Log("Press any 2 SHIFT-ALT-CTRL keys to add mark to logfile");
            botStart = new BotEvents.OnBotStartDelegate(OnBotStart);
            BotEvents.OnBotStart += botStart;
            botStop = new BotEvents.OnBotStopDelegate(OnBotStop);
            BotEvents.OnBotStop += botStop;
            haveInitialized = true;
            base.Initialize();
        }

        public override void Dispose()
        {
            if ( haveInitialized )
            {
                Logging.WriteDiagnostic("[LOGMARK] Dispose");
                haveInitialized = false;
                BotEvents.OnBotStart -= botStart;
                BotEvents.OnBotStop -= botStop;
                Lua.Events.DetachEvent("MODIFIER_STATE_CHANGED", HandleModifierStateChanged);
            }
            base.Dispose();
        }

        private void OnBotStart(EventArgs args)
        {
            Logging.WriteDiagnostic("[LOGMARK] OnBotStart");
            Lua.Events.AttachEvent("MODIFIER_STATE_CHANGED", HandleModifierStateChanged);
        }

        private void OnBotStop(EventArgs args)
        {
            Logging.WriteDiagnostic("[LOGMARK] OnBotStop");
            Lua.Events.DetachEvent("MODIFIER_STATE_CHANGED", HandleModifierStateChanged);
        }

        public override void Pulse()
        {
            ;
        }


        private int downCount = 0;
        private int lastCount = 0;
        private long markCount = 0;


        private void HandleModifierStateChanged(object sender, LuaEventArgs args)
        {
            // if key released
            if (Convert.ToInt32(args.Args[1]) != 1)
            {
                if (downCount > 0)
                    downCount--;
            }
            else
            {
                // else, key pressed
                downCount++;
                if (downCount == 2 && lastCount == 1)    // check we have 2 modifiers and previously only 1
                {
                    Log(Colors.OrangeRed, "Mark # {0} added to logfile", ++markCount);
                }
            }
            lastCount = downCount;
        }

        public void Log(string msg, params object[] args)
        {
            Log(System.Windows.Media.Colors.Gold, msg, args);
        }

        private long lineCount = 0;
        public void Log(System.Windows.Media.Color clr, string msg, params object[] args)
        {
            Logging.Write(LogLevel.Normal, clr, "[LOGMARK] " + msg + (++lineCount % 2 == 0 ? "" : " "), args);
        }

    }
}
